

/*******************************************************************************
* Applikation Schirennlauf:                                                    *
*                                                                              *
********************************************************************************
* Beschreibung:                                                                *
*                                                                              *
* ...die Ampel steht auf Rot (die Piste ist fuer Schirennlaeufer gesperrt).    *
* Die rote Lampe der Ampel wird mittels P7.0 vom Mikrocontroller angesteuert.  *
* Die gruene Lampe der Ampel wird mittels P7.1 vom Mikrocontroller angesteuert.*
*                                                                              *
* Der Name des Schirennlaeufers wird erfasst.                                  *
* Der externe Interrupt fuer Port 2.0 wird freigegeben.                        *
* Die Ampel wechselt auf Gruen (Piste frei).                                   *                                                                              *
*                                                                              *
* Der Schilaeufer startet, die Zeitmessung beginnt:                            *
* Mittels positiver Flanke an Port 2.0 wird ein externer Interrupt ausgeloest. *
* = START-SIGNAL                                                               *
* Dieser Interrupt startet Timer 0 und schaltet die Ampel auf Rot.             *
*                                                                              *
* Der Schilaeufer erreicht das Ziel, die Zeitmessung stoppt:                   *
* Mittels positiver Flanke an Port 2.1 wird ein externer Interrupt ausgeloest  *
* = STOPP-SIGNAL                                                               *
* Dieser Interrupt stoppt Timer 0.                                             *
* Mittels globaler Variable schilaeufer_ziel_erreicht, wird dem Hauptprogramm  *
* mitgeteilt, dass der Schilaeufer das Ziel erreicht hat und die Zeitmessung   *
* abgeschlossen ist.                                                           *
*                                                                              *
* u.s.w. ...                                                                   *
*                                                                              *
*                                                                              *
*******************************************************************************/


#include <REG167.H>   /* Register Definitionen fr Mikrocontroller C167        */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "serio.h"

#define MAXNAME 15    /* maximale Anzahl von Zeichen-1 fr Schi-Laeufer-Namen  */

#define TRUE 1
#define FALSE 0

// Bitname fuer das Port-Richtungsbit AUSGANG_ROT definieren:
// AUSGANG_ROT = 0 bedeutet: Port 7.0 ist Eingang (hochohmiger Zustand)
// AUSGANG_ROT = 1 bedeutet: Port 7.0 ist Ausgang (PUSH-PULL)
// PUSH-PULL bedeutet: es wird aktiv 0 Volt oder 5 Volt ausgegeben
// (je nach Inhalt vom Portlatch P7.0)
sfrbit AUSGANG_ROT _atbit(DP7,0);

// Bitname fuer das Port-Richtungsbit AUSGANG_GRUEN definieren:
sfrbit AUSGANG_GRUEN _atbit(DP7,1);

// das bit ROT ist der Zustand der aktiv ( 0 oder 5 Volt) am Pin
// AUSGANG_ROT = P7.0 ausgegeben wird:
sfrbit ROT _atbit(P7,0);

// das bit GRUEN ist der Zustand der aktiv ( 0 oder 5 Volt) am Pin
// AUSGANG_GRUEN = P7.1 ausgegeben wird:
sfrbit GRUEN _atbit(P7,1);



// Definition eines Knoten vom binaeren Baum:
typedef struct knoten
{
	char name[MAXNAME];    /* Name des Schi-Laeufers         */
	int  zeit;             /* gelaufene Zeit in [s]          */
	struct knoten *links;  /* linker Nachkomme               */
	struct knoten *rechts; /* rechter Nachkomme              */
} KNOTEN;


/*===================== Prototypen ===========================================*/
void main (void); /* Hauptprogramm = Interruptservice-Routine mit             */
		  /* Interruptvektor (Einsprungadresse) auf Adresse 0         */
		  /* Ereignis welches zum Aufruf fhrt: RESET                 */

void init_capcom(void);      /* Initialisierung der externen Interrupts       */
void init_timer_0(void);     /* Initialisierung von Timer 0                   */
void init_interrupt(void);   /* Initialisierung der Interrupt-Prioritaeten    */

KNOTEN *einordnen(KNOTEN *knot_zeig, char *name);
void drucke_binaeren_baum(KNOTEN *knot_zeig, int *nr);
KNOTEN *knoten_fuer_binaeren_baum_anlegen(void);

void externen_interrupt_fuer_start_freigeben(void);
/*============================================================================*/

/*===================== Prototypen Interruptservice-Routinen =================*/
interrupt (0x10) void externer_interrupt_START (void);  /* Signal an Port 2.0 */
interrupt (0x11) void externer_interrupt_STOPP (void);  /* Signal an Port 2.1 */
interrupt (0x20) void timer_0_SEKUNDENINTERRUPT (void);
/*============================================================================*/

/*== globale Variablen zur Kommunikation: Hauptprogramm <-> Interrupts =======*/
char schilaeufer_ziel_erreicht;
int zeit_in_sekunden;
bit start_signal_freigabe;
bit stopp_signal_freigabe;
/*============================================================================*/


/*============================================================================*/
void main (void)
{
 KNOTEN *wurzel;       // Pointer der auf den binaeren Baum zeigt anlegen
 char wer[MAXNAME];    // Platz fuer den Namen des Schirennlaeufers schaffen
 int i=1, platz;

 ROT = TRUE;           // Ampelinitialisierung bei Programmstart
 GRUEN = FALSE;        // Ampelinitialisierung bei Programmstart
 AUSGANG_ROT = TRUE;   // Port 7.0 = AUSGANG (PUSH-PULL), P7.0 ist jetzt high
 AUSGANG_GRUEN = TRUE; // Port 7.1 = AUSGANG (PUSH-PULL), P7.0 ist jetzt low

 start_signal_freigabe = FALSE;
 stopp_signal_freigabe = FALSE;

 wurzel=NULL;          // Anfang des binaeren Baums festlegen

 init_serio();         // Initialisierung der seriellen Schnittstelle aufrufen
 init_capcom();        // Initialisierung der externen Interrupts
 init_timer_0();       // Initialisierung von Timer 0
 init_interrupt();     // Initialisierung der Interrupt-Prioritaeten

 IEN=1;                // Interruptsystem global freigeben

 while (1)
 {
	do
	{
		printf("\n\nName des am START stehenden Schilaeufers = ? ");
		scanf("%s",wer); fflush(stdin);

		if (wer[0]!='*')
		{
			schilaeufer_ziel_erreicht='n'; // n...nein
			zeit_in_sekunden=0;

			externen_interrupt_fuer_start_freigeben();

			printf("\n");
			printf("\n%5u Sekunden, ",zeit_in_sekunden);
			printf(" ( aktueller Zwischenstand Laeufer: ");
			printf("%15s )",wer);
			while (schilaeufer_ziel_erreicht == 'n')   // warten, bis die
			{                                          // Zeitmessung
								   // beendet ist !!!
				printf("\r%5u",zeit_in_sekunden);

			} /* ende while */
			printf("\n");

			wurzel=einordnen(wurzel,wer);
			printf("\n\nZwischenstand nach %d Laeufer(n)\n",i++);

		} /* ende if */

		else
			printf("\n\nEndergebnis:\n");

		printf("\n%12s %-15s %12s [Sekunden]\n","Platz","Name","Zeit");
		platz=1;
		drucke_binaeren_baum(wurzel,&platz);

	} while(wer[0]!='*');

	_int166(0); // SRST...Software Reset
		    // Aus der Sicht eines PC-Programmierers eine ungewoehnliche
		    // Art und Weise mittels malloc reservierten Speicher freizugeben
 } /* ende while (1) */

} /* ende main */
/*============================================================================*/

/*============================================================================*/
void init_capcom(void)
{
 CCM0 = 0x0011;   /* 0000 0000 0001 0001 B                                    */
		  /* |||| |||| |||| 0001: CAPCOM1_Kanal_0                     */
		  /* |||| |||| |||| ||01: pos. Flanke auf P2.0 = Signal START */
		  /* |||| |||| |||| |0: capture mod = externer Interrupt      */
		  /* |||| |||| |||| 0: zugeordneter Timer egal                */
		  /* |||| |||| 0001: CAPCOM1_Kanal_1                          */
		  /* |||| |||| ||01: positive Flanke auf P2.1 = Signal STOPP  */
		  /* |||| |||| |0: capture mod = externer Interrupt           */
		  /* |||| |||| 0: zugeordneter Timer egal                     */
		  /* |||| 0000: CAPCOM1_Kanal_2                               */
		  /* 0000: CAPCOM1_Kanal_3                                    */

} /* ende init_capcom */
/*============================================================================*/

/*============================================================================*/
void init_timer_0(void)
{

 T01CON = 0x0006;  /* xxxx xxxx x0xx 0110 B                                   */
		   /*            |   |110: T0I:Timer/Counter 0 Input Selection*/
		   /*            |   |          Teiler fuer fCPU = 512        */
		   /*            |   0: T0M: Timer/Counter-Mode: 0=Timer Mode */
		   /*            0: Timer/Counter 0 Run Control: Timer steht  */

 T0REL  = 26474;  /* Reload-Wert fuer Timer 0 Hochlauf fuer Periode=1 Sekunde */
		  /* PT0 = ( (65536-T0REL) * 2 hoch(T0I+3) ) / fCPU =
		  /*     = ( (65536-26474) * 2 hoch(6+3)   ) / 20 000 000 = 1 */
 T0=T0REL;

} /* ende init_timer */
/*============================================================================*/

/*============================================================================*/
void init_interrupt(void)
{
  // Interrupt Prioritaet Timer 0 (Sekundensignal): ILVL=10,GLVL=0
 T0IC  = 0x0068;   /* xxxx xxxx 01 1010 00 B                                  */
		   /*           || |||| 00: Group Level GLVL=0                */
		   /*           || 1010: Interrupt Priority Level ILVL=10     */
		   /*           |1: Interrupt Enable Control Bit setzen       */
		   /*           0: Interrupt Request Flag loeschen            */

 // Interrupt Prioritaet Signal START: ILVL=14,GLVL=0
 CC0IC = 0x0078;   /* xxxx xxxx 01 1110 00 B, IR=0, IE=1, ILVL=14, GLVL=0     */

 // Interrupt Prioritaet Signal STOPP: ILVL=14,GLVL=1
 CC1IC = 0x0079;   /* xxxx xxxx 01 1110 01 B, IR=0, IE=1, ILVL=14, GLVL=1     */


} /* ende init_interrupt */
/*============================================================================*/



/*============================================================================*/
KNOTEN *einordnen(KNOTEN *knot_zeig, char *name)
{

 if (knot_zeig==NULL)
 {
	knot_zeig=knoten_fuer_binaeren_baum_anlegen();
	strcpy(knot_zeig->name,name);
	knot_zeig->zeit=zeit_in_sekunden;
	knot_zeig->links=knot_zeig->rechts=NULL;
 } /* ende if */

 else
	if (((zeit_in_sekunden-knot_zeig->zeit)<0) ||
	((zeit_in_sekunden==knot_zeig->zeit) && (strcmp(name,knot_zeig->name)<0)))
		knot_zeig->links=einordnen(knot_zeig->links,name);
	else
		knot_zeig->rechts=einordnen(knot_zeig->rechts,name);

 return(knot_zeig);
} /* ende *einordnen */
/*============================================================================*/

/*============================================================================*/
void drucke_binaeren_baum(KNOTEN *knot_zeig, int *nr)
{
 if (knot_zeig!=NULL)
 {
	drucke_binaeren_baum(knot_zeig->links,nr);
	printf("%12d %-15s %12d\n",(*nr)++,knot_zeig->name,knot_zeig->zeit);
	drucke_binaeren_baum(knot_zeig->rechts,nr);
 } /* ende if */
} /* ende drucke_binaeren_baum */
/*============================================================================*/

/*============================================================================*/
KNOTEN *knoten_fuer_binaeren_baum_anlegen(void)
{
 KNOTEN *return_wert;

 return_wert=((KNOTEN *)malloc(sizeof(KNOTEN)));

 if (return_wert == NULL)
    printf("\n\n!!!ERROR: KEIN SPEICHERPLATZ VORHANDEN !!!"), _int166(0);

 return return_wert;

} /* ende KNOTEN *knoten_fuer_binaeren_baum_anlegen */
/*============================================================================*/

/*============================================================================*/
void externen_interrupt_fuer_start_freigeben(void)
{
 // Aufgabe: Start des Schilaufers vorbereiten: Ampel = Gruen

 start_signal_freigabe = TRUE;

 ROT   = FALSE;  /* Piste frei                                                */
 GRUEN = TRUE;   /* Piste frei                                                */

} /* ende externen_interrupt_fuer_start_freigeben */
/*============================================================================*/

/*============================================================================*/
interrupt (0x10) void externer_interrupt_START (void)
{
 // Aufgabe: Zeitmessung (Timer 0) starten und Ampel auf ROT schalten

 if (start_signal_freigabe)
 {
	T0R = 1; /* Timer 0 Run-Bit gesetzt -> Timer 0 laeuft (Zeitmessung laeuft)   */

	GRUEN = FALSE;  /* Piste fuer den naechsten Laeufer gesperrt                 */
	ROT   = TRUE;   /* Piste fuer den naechsten Laeufer gesperrt                 */

	start_signal_freigabe = FALSE;
	stopp_signal_freigabe = TRUE;
 } /* ende if */

} /* ende externer_interrupt_START */
/*============================================================================*/

/*============================================================================*/
interrupt (0x11) void externer_interrupt_STOPP (void)
{
 // Aufgabe: Timer_0 wird gestoppt.
 // mittels globaler Variable schilaeufer_ziel_erreicht wird dem Hauptprogramm
 // mitgeteilt, dass der Schilaeufer das Ziel erreicht hat, und die Zeitmessung
 // beendet ist.

 if (stopp_signal_freigabe)
 {
	T0R = 0; /* Timer 0 Run-bit geloescht -> Timer 0 steht (Zeitmessung beendet) */
	T0  = T0REL; /* Timer 0 Zaehlregister loeschen                               */

	stopp_signal_freigabe = FALSE;

	schilaeufer_ziel_erreicht = 'j'; /* j...ja...Ziel erreicht !!!               */
 } /* ende if */

} /* ende externer_interrupt_STOPP */
/*============================================================================*/

/*============================================================================*/
interrupt (0x20) void timer_0_SEKUNDENINTERRUPT (void)
{
 zeit_in_sekunden++;
} /* ende timer_0_SEKUNDENINTERRUPT */
/*============================================================================*/
