; Programm: PRG4.ASM
LIST P=16C71
        
; Symbolzuweisung
        W        EQU 00h
        f        EQU 01h

        RTCC_L   EQU 01h
        RTCC_H   EQU 11h

        STATUS_R EQU 03h
        Port_A   EQU 05h
        Port_B   EQU 06h

        INTCON_R EQU 0Bh
        OPTION_R EQU 81h
        Tris_A   EQU 85h
        Tris_B   EQU 86h
        ADCON1_R EQU 88h
        
        #DEFINE Page_1 STATUS_R,5
        #DEFINE T0IF   INTCON_R,2

; Zuordnung des Resetvektors
ORG 000h
        Goto Init

; Zuordnung des Interruptvektors
ORG 04h
        INCF RTCC_H,f
        BCF  T0IF
        RETFIE
        

; Programmzuweisung
ORG 010h

Init:              
        MOVLW b'10100000'
        MOVWF INTCON_R   ; Init. des INTCON-Reg.
        ; T0IE = 1 .. Interrupt Enable Bit EIN
        ; GIE  = 1 .. Global Interrupt EIN
                
        BSF   Page_1
        ; Umschalten auf 1. Seite
        ; um OPTION-Register, Tris_A/B & ADCON zu
        ; konf. Der Assembler gibt einen Meldung
        ; (4x) aus, dass bei 8Xh-Adressierung das
        ; oberste Bit (also der 8-ter) wegge-
        ; schnitten wurde, dieses geschieht
        ; dadurch, dass bei der Register-
        ; adressierung nur 7 Bit (0-127)
        ; moeglich ist, daher muss auf die 
        ; 1. Seite umgeschaltet werden, nachher
        ; wieder auf Seite 0 zurueckschalten nicht 
        ; vergessen.

        MOVLW b'10000011'
        MOVWF OPTION_R    ; Init. des OPTION-Reg.
        ; /RBPU = 1 .. Pull Up AUS
        ; RTS = 0 Eingang = f osc/4
        ; RTE = 0 L-H-Flanke
        ; PSA = 0 Vorteilerzuweisung an f osc/4
        ; PS210 = 011 Vorzaehler-Rate = 16
        ; Auf Seite 1 schalten!!
               
        MOVLW b'00000000'
        MOVWF Tris_B      ; Init. d. IO-Dir-Reg.PB
        ; alle AUSgabe
        ; Auf Seite 1 schalten!!

        MOVLW b'00011111'
        MOVWF Tris_A      ; Init. d. IO-Dir-Reg.PA
        ; Port A 0-4 EINgaenge
        ; Auf Seite 1 schalten!!
        
        MOVLW b'00000011'
        MOVWF ADCON1_R    ; Init. des ADCON1-Reg.
        ; PA0-3 als digital E/A

        BCF   Page_1   ; Umschalten auf Seite 0

        CLRF  Port_B   ; Loeschen des Port_B-Reg.
        CLRF  RTCC_L   ; Loeschen des RTCC_L 
        CLRF  RTCC_H   ; Loeschen des RTCC_H

Prog:   SWAPF RTCC_L,W
        SUBWF RTCC_H,W
        MOVWF Port_B
        GOTO  Prog

; -----------------------------------------------
; Dieses Demoprogramm soll den Umgang mit dem
; Zaehlerinterrupt (T0IF) zeigen.
;   Es soll einen 16-Bit Zaehler mit Hilfe des
; Interrupts realsiert werden, wobei RTCC_L
; das eigentliche RTCC-Register (01h) und das
; RTCC_H das hoeherwertige Byte des Wortes ist,
; welche von einem Register (General Purpose
; Register) (Adresse 11h) verwendet wird.
;   Das Hauptprogramm ist eigentlich in diesem
; Fall unrelevant. Hier koennte jedoch eine Ver-
; arbeitung der zwei Register RTCC_H & RTCC_L
; stehen. Es sollte jedoch einen so hohen 
; Verteiler des RTCC und somit niedrige Zyklenzeit
; gewaehlt werden, so da keine Aenderung der 
; beiden Register in der Verarbeitung stattfindet.
;   In diesem Programmbeispiel wurde fr die 
; Simulation die RTCC-Quelle das Viertel des 
; Oszillatorfrequenz gewhlt.
; .
; Simulator-Bericht:
; -PC- --- Befehl ---  T_H T_L -INTCON- STACK Bem
; 001F SWAPF RTCC_L,W  00h FFh 10100000  -- 
; 0020 SUBWF RTCC_H,W  00h FFh 10100000  --
; 0004 INCF  RTCC_H,f  00h 00h 00100100  21h   !
; 0005 BCF   T0IF      01h 00h 00100100  21h
; 0006 RETFIE          01h 00h 00100000  21h
; 0021 MOVWF Port_B    01h 00h 10100000  --
; 0022 GOTO  Prog      01h 00h 10100000  --
; Legende: PC = Program Counter
; T_H = RTCC_H (11h)   T_L = RTCC_L (01h)
END
