#include <alloc.h>
#include <io.h>
#include <fcntl.h>
#include <mem.h>
#include "voc.h"

Voc::Voc(char *FileName) : Sound()
{
  Handle=open(FileName,O_RDONLY | O_BINARY);
  if (Handle<0) ErrorMsg(NOFILE);

  FN=FileName;
}

// Lesenprozedur fr VOC-Dateien ...
char Voc::Read()
{
  VOCHEADER VocHeader;
  BLOCKHEADER Header;
  BYTE LastBlock=0,rueck,huge *SoundDaten,huge *Zeiger,huge *RepeatBlocks=NULL;
  WORD RepeatValue=0,SampleRate=0;
  DWORD Laenge=1;

  SoundDaten=Zeiger=(BYTE huge *)farmalloc(1);

  read(Handle,&VocHeader,sizeof(VOCHEADER));
  if (memcmp(&VocHeader.FileID,"Creative Voice File\x1a",sizeof(VocHeader.FileID))) return NOVOC;
			   // berprfen ob wirklich VOC-Format vorliegt
  do
  {
     // Kopf eines Blocks lesen
    if (!read(Handle,&Header,sizeof(BLOCKHEADER))) break; // wenn Ende der Datei erreicht ist, Schleife abbrechen

    switch(Header.BlockTyp)
    {
      case 0x1:
      {
	VOICE_DATA Block;

	read(Handle,&Block,sizeof(VOICE_DATA));
	if (LastBlock!=8) SampleRate=1e6/(256-Block.TimeConstant);
	// wenn nicht bereits EXTENDED-Block gelesen, SampleFrequenz errechnen
	DWORD len=GetBlockLength(Header.BlockLaenge);
						  // Lnge des Blocks ermitteln
	SoundDaten=(BYTE huge *)farrealloc(SoundDaten,Laenge+len);
	if (!SoundDaten) return NOMEM;
	Zeiger=SoundDaten+Laenge;
	Laenge+=len;

	while (len>60000l)     // solange len grer als 60000, Schleife ausfhren
	{
	  len-=60000l;
	  read(Handle,Zeiger,60000u); // 60KB von Datei lesen
	  Zeiger+=60000l;         // Pufferzeiger um 60000 erhhen
	}
	read(Handle,Zeiger,(unsigned)len); // Rest einlesen

	Zeiger=SoundDaten;
	LastBlock=1;

	break;
      }
      case 0x2:    // VOICE_CONTINUATION Block - Erweiterung zu VOICE_DATA
      {
	DWORD len=GetBlockLength(Header.BlockLaenge);

	SoundDaten=(BYTE huge *)farrealloc(SoundDaten,Laenge+len);
	if (!SoundDaten) return NOMEM; // Gre des Blocks auf dem Heap erhhen

	Zeiger=SoundDaten+Laenge;

//	read(Handle,Zeiger,len); // am Ende des Puffers neue Daten hinzufgen

	DWORD Len=len;
	while (Len>60000l)     // solange len grer als 60000, Schleife ausfhren
	{
	  Len-=60000l;
	  read(Handle,Zeiger,60000u); // 60KB von Datei lesen
	  Zeiger+=60000l;         // Pufferzeiger um 60000 erhhen
	}
	read(Handle,Zeiger,(unsigned)Len); // Rest einlesen

	Laenge+=len;             // Gesamt-Lnge anpassen
	Zeiger=SoundDaten;
	LastBlock=2;

	break;
      }
      case 0x3:
      {
	SILENCE_BLOCK Block;

	read(Handle,&Block,sizeof(SILENCE_BLOCK));

	WORD Bytes=SampleRate/(1e6/(256-Block.SampleRate))*Block.SilenceDuration;
	SoundDaten=(BYTE huge *)farrealloc(SoundDaten,Laenge+Bytes);
	Zeiger=SoundDaten+Laenge;

	_fsetmem(Zeiger,Bytes,0x0);

	LastBlock=3;

	break;
      }
      case 0x4: // MARKER-Block fr Synchronisation
      {
	lseek(Handle,tell(Handle)+2,SEEK_SET); // zum nchsten Block springen
	LastBlock=4;

	break;
      }
      case 0x5: // MESSAGE-Block; enthlt ASCII-Kommentare
      {
	DWORD len=GetBlockLength(Header.BlockLaenge);
	lseek(Handle,tell(Handle)+len,SEEK_SET); // zum nchsten Block springen
	LastBlock=5;

	break;
      }
      case 0x6:
      {
	REPEAT_LOOP Block;      // alle nachfolgenden Blcke bis zum END_REPEAT
				// Block n+1 mal wiederholen
	read(Handle,&Block,sizeof(REPEAT_LOOP));
	RepeatValue=Block.RepeatValue;
	LastBlock=6;
	RepeatBlocks=Zeiger;

	break;
      }
      case 0x7:
      {
	LastBlock=7;

	break;
      }
      case 0x8:
      {
	EXTENDED Block;   // anschlieend MUSS VOICE_DATA Block folgen

	read(Handle,&Block,sizeof(EXTENDED));
	SampleRate=256e6/(65532u-Block.TimeConstant);
	LastBlock=8;
		     // bewirkt, da SampleRate im folgenden VOICE_DATA Block
	break;       // nicht neu berechnet wird
      }
    }
  }
  while(Header.BlockTyp!=0x0);  // solange kein TERMINATOR-Block gefunden
  close(Handle);                // Schleife ausfhren

  SetSoundSampleRate(SampleRate);
  SetSoundLength(Laenge);
  SetSoundBuffer(SoundDaten);

  return 0;
}